import numpy as np
import torch
import torch.backends.cudnn as cudnn
from util.test_util import CodedCamera
from pathlib import Path

from model import ApertureMask, PixelMask


def main(args):

    ''' Create Dir for Save '''
    result_dir = Path("Dst")
    result_dir.mkdir(exist_ok=True)

    ''' CPU or Cuda'''
    device = torch.device(args.device)
    if 'cuda' in args.device:
        torch.cuda.set_device(device)

    ''' MODEL LOADING '''
    aperture_mask = []
    pixel_mask = []
    for s in range(args.shots_num):
            tmp_mask = ApertureMask(args)
            tmp_mask.load_state_dict(torch.load("%s/Model/ApertureMask%01d_Epoch%03d.pth"%(args.model_data, s+1,args.epoch)))
            aperture_mask.append(tmp_mask.to("cuda"))


    for n in range(args.pix_mask_size**2):
            tmp_mask = PixelMask(args)
            tmp_mask.load_state_dict(torch.load("%s/Model/ForwardPixelMask/ForwardPixelMask%01d_Epoch%03d.pth"%(args.model_data,n+1,args.epoch)))
            pixel_mask.append(tmp_mask.to("cuda"))

    cudnn.benchmark = True

    print('Making coded image...')


    with torch.no_grad():

        save_dir_ = result_dir.joinpath(args.scene)
        save_dir_.mkdir(exist_ok=True)

        coded_camera = CodedCamera(args, save_dir_, device)

        if args.coded_img_path is None:
            print("Scene, {}".format(args.scene))

            """load original views from test loader."""
            data = coded_camera.prepare_data(args.src_data_path, args.data_for)
            data = torch.from_numpy(data[None,...].astype(np.float32)).clone()


            """Make coded image from 5x5 original views"""
            coded_camera.make_coded_img(data, aperture_mask, pixel_mask)
            

if __name__ == '__main__':
    from option import args

    main(args)
