import torch
import torch.nn.functional as F
import torch.nn as nn

class ApertureMask(nn.Module):
    def __init__(self, args):
        super(ApertureMask, self).__init__()
        self.ch_in = args.angRes*args.angRes
        self.cn01 = nn.Conv2d(self.ch_in,1,1,padding=0,bias=False)

    def forward(self, f0):
        f1 = self.cn01(f0)
        return f1
        

class PixelMask(nn.Module):
    def __init__(self, args):
        super(PixelMask, self).__init__()
        self.ch_in = args.shots_num
        self.cn01 = nn.Conv2d(self.ch_in,1,1,padding=0,bias=False)

    def forward(self, f0):
        f1 = self.cn01(f0)
        return f1


class PixelSuffuleReconstruct(nn.Module):
    def __init__(self, args):
        super(PixelSuffuleReconstruct, self).__init__()
        self.ps = nn.PixelShuffle(args.pix_mask_size)
        self.ch_out = args.feature_ch*args.feature_z_num
        self.conv1 = nn.Conv2d(64,64,1,padding=0)
        self.conv2 = nn.Conv2d(64,128,3,padding=1)
        self.conv3 = nn.Conv2d(128,128,3,padding=1)
        self.conv4 = nn.Conv2d(128,128,3,padding=1)
        self.conv5 = nn.Conv2d(128,256,3,padding=1)
        self.conv6 = nn.Conv2d(4,16,5,padding=2)
        self.conv7 = nn.Conv2d(16,64,5,padding=2)
        self.conv8 = nn.Conv2d(64,self.ch_out,5,padding=2)

    def forward(self,f0):
        f1 = self.conv1(f0)
        f2 = self.conv2(f1)
        f3 = self.conv3(f2)
        f4 = self.conv4(f3)
        f5 = self.conv5(f4)
        f6 = self.conv6(self.ps(f5))
        f7 = self.conv7(f6)
        f8 = self.conv8(f7)
        return f8


class VDSR(torch.nn.Module):
    def __init__(self, args, ch_mid):
        super(VDSR, self).__init__()
        self.ch_in = args.feature_ch*args.feature_z_num # 8 x 13
        self.ch_mid = ch_mid # 64
        self.ch_out = args.feature_ch*args.feature_z_num # 8 x 13
        self.l1  = torch.nn.Conv2d(in_channels=self.ch_in, out_channels=self.ch_mid, kernel_size=3, stride=1, padding=1)
        self.l2  = torch.nn.Conv2d(in_channels=self.ch_mid, out_channels=self.ch_mid, kernel_size=3, stride=1, padding=1)
        self.l3  = torch.nn.Conv2d(in_channels=self.ch_mid, out_channels=self.ch_mid, kernel_size=3, stride=1, padding=1)
        self.l4  = torch.nn.Conv2d(in_channels=self.ch_mid, out_channels=self.ch_mid, kernel_size=3, stride=1, padding=1)
        self.l5  = torch.nn.Conv2d(in_channels=self.ch_mid, out_channels=self.ch_mid, kernel_size=3, stride=1, padding=1)
        self.l6  = torch.nn.Conv2d(in_channels=self.ch_mid, out_channels=self.ch_mid, kernel_size=3, stride=1, padding=1)
        self.l7  = torch.nn.Conv2d(in_channels=self.ch_mid, out_channels=self.ch_mid, kernel_size=3, stride=1, padding=1)
        self.l8  = torch.nn.Conv2d(in_channels=self.ch_mid, out_channels=self.ch_mid, kernel_size=3, stride=1, padding=1)
        self.l9  = torch.nn.Conv2d(in_channels=self.ch_mid, out_channels=self.ch_mid, kernel_size=3, stride=1, padding=1)
        self.l10 = torch.nn.Conv2d(in_channels=self.ch_mid, out_channels=self.ch_mid, kernel_size=3, stride=1, padding=1)
        self.l11 = torch.nn.Conv2d(in_channels=self.ch_mid, out_channels=self.ch_mid, kernel_size=3, stride=1, padding=1)
        self.l12 = torch.nn.Conv2d(in_channels=self.ch_mid, out_channels=self.ch_mid, kernel_size=3, stride=1, padding=1)
        self.l13 = torch.nn.Conv2d(in_channels=self.ch_mid, out_channels=self.ch_mid, kernel_size=3, stride=1, padding=1)
        self.l14 = torch.nn.Conv2d(in_channels=self.ch_mid, out_channels=self.ch_mid, kernel_size=3, stride=1, padding=1)
        self.l15 = torch.nn.Conv2d(in_channels=self.ch_mid, out_channels=self.ch_mid, kernel_size=3, stride=1, padding=1)
        self.l16 = torch.nn.Conv2d(in_channels=self.ch_mid, out_channels=self.ch_mid, kernel_size=3, stride=1, padding=1)
        self.l17 = torch.nn.Conv2d(in_channels=self.ch_mid, out_channels=self.ch_mid, kernel_size=3, stride=1, padding=1)
        self.l18 = torch.nn.Conv2d(in_channels=self.ch_mid, out_channels=self.ch_mid, kernel_size=3, stride=1, padding=1)
        self.l19 = torch.nn.Conv2d(in_channels=self.ch_mid, out_channels=self.ch_mid, kernel_size=3, stride=1, padding=1)
        self.l20 = torch.nn.Conv2d(in_channels=self.ch_mid, out_channels=self.ch_out, kernel_size=3, stride=1, padding=1)

    def forward(self, x):

        y  = F.relu(self.l1(x))
        y  = F.relu(self.l2(y))
        y  = F.relu(self.l3(y))
        y  = F.relu(self.l4(y))
        y  = F.relu(self.l5(y))
        y  = F.relu(self.l6(y))
        y  = F.relu(self.l7(y))
        y  = F.relu(self.l8(y))
        y  = F.relu(self.l9(y))
        y = F.relu(self.l10(y))
        y = F.relu(self.l11(y))
        y = F.relu(self.l12(y))
        y = F.relu(self.l13(y))
        y = F.relu(self.l14(y))
        y = F.relu(self.l15(y))
        y = F.relu(self.l16(y))
        y = F.relu(self.l17(y))
        y = F.relu(self.l18(y))
        y = F.relu(self.l19(y))
        y = self.l20(y)
        return y + x