import argparse

parser = argparse.ArgumentParser()

"""Parameters for datasets"""
parser.add_argument('--data_for',       type=str,   default='test',        help='')
parser.add_argument('--src_data_path',  type=str,   default='data/datasets/', help='source data path')


"""Parameters for camera side coding"""
parser.add_argument('--batch_size', type=int, default=4)
parser.add_argument("--patch_size", type=int, default=128, help="patch size")
parser.add_argument('--epoch', default=120, type=int, help='Epoch to run')
parser.add_argument("--shots_num", type=int, default=4, help = "shots num")
parser.add_argument("--angRes", type=int, default=5, help="angular resolution")
parser.add_argument("--pix_mask_size", type=int, default=8, help = "pixel mask row size (default : 8)")
parser.add_argument("--light_kind_num", type=int, default=6, help = "augmentation light kind num")
parser.add_argument("--noise", type=float, default=0.005, help = "noise")

parser.add_argument('--data_name', type=str, default='ALL', help='EPFL, HCI_new, HCI_old, INRIA_Lytro, Stanford_Gantry, ALL(of Five Datasets)')
parser.add_argument('--path_for_train', type=str, default='../../data/BasicLFSR/data_for_training/')
parser.add_argument('--train_data_var', type=float, default=0.25)

parser.add_argument('--path_for_test', type=str, default='data/data_for_test/')
parser.add_argument("--model_data", type=str, default="pretrained_model", help="model data")
parser.add_argument("--channel_num", type=int, default=1, help = "test chanel (1 or 3)")
parser.add_argument('--save_image', default=True, type = bool, help='save image when test ')

parser.add_argument('--n_steps', type=int, default=15, help='number of epochs to update learning rate')
parser.add_argument('--num_workers', type=int, default=2, help='num workers of the Data Loader')
parser.add_argument('--local_rank', dest='local_rank', type=int, default=0, )
parser.add_argument('--device', type=str, default='cuda:0')


"""Parameters for feature volume"""
parser.add_argument("--feature_ch", type = int, default = 8, help = "coorinates's features")
parser.add_argument("--feature_z_num", type = int, default = 13, help = "z num in feature output")


"""Parameters for orthogonal nerf"""
# training options
parser.add_argument("--netdepth", type=int, default=4, 
                    help='layers in network')
parser.add_argument("--netwidth", type=int, default=64, 
                    help='channels per layer')
parser.add_argument("--lrate", type=float, default=5e-4, 
                    help='learning rate')
parser.add_argument("--lrate_decay", type=int, default=250, 
                    help='exponential learning rate decay (in 1000 steps)')
parser.add_argument("--chunk", type=int, default=64*64*25, 
                    help='number of rays processed in parallel, decrease if running out of memory')
parser.add_argument("--netchunk", type=int, default=64*64*10*25, 
                    help='number of pts sent through network in parallel, decrease if running out of memory')
parser.add_argument("--netchunk_test", type=int, default=64*64*8*25, 
                    help='number of pts sent through network in parallel, decrease if running out of memory')
parser.add_argument("--lindisp", action='store_true', 
                    help='sampling linearly in disparity rather than depth')

# rendering options
parser.add_argument("--N_samples", type=int, default=16, 
                    help='number of coarse samples per ray')
parser.add_argument("--N_samples_test", type=int, default=32, 
                    help='number of coarse samples per ray')
parser.add_argument("--perturb", type=float, default=1.,
                    help='set to 0. for no jitter, 1. for jitter')
parser.add_argument("--use_viewdirs", action='store_false', 
                    help='use full 5D input instead of 3D')
parser.add_argument("--i_embed", type=int, default=-1, 
                    help='set 0 for default positional encoding, -1 for none')
parser.add_argument("--multires", type=int, default=4, 
                    help='log2 of max freq for positional encoding (3D location)')
parser.add_argument("--multires_views", type=int, default=2, 
                    help='log2 of max freq for positional encoding (2D direction)')
parser.add_argument("--raw_noise_std", type=float, default=1e-1, 
                    help='std dev of noise added to regularize sigma_a output, 1e0 recommended')
parser.add_argument("--near", type = float, default = -3., help = "min z plane value")
parser.add_argument("--far", type = float, default = 3., help = "max z plane value")
parser.add_argument("--retraw", type = int, default = 0)

parser.add_argument("--z_factor", type = float, default=1., help = "z plane factor")
parser.add_argument("--no_reload", action = "store_true", help = "if true, load checkpoint")
parser.add_argument("--cut_edge", type=int, default=1, help = "if true, cut edge in training time and test time")

# test option
parser.add_argument("--test_mode", type = int, default=0, help="if 1, test mode, if 2, test_nv, if 3, circle")
parser.add_argument("--view_range", type=int, default=7, help = "new view num")
parser.add_argument("--depthMax", type = float, default=3., help = "depth max range")
parser.add_argument("--depthMin", type = float, default=-3., help = "depth min range")
parser.add_argument("--newview_resolution", type=int, default=13, help = "new view num")
parser.add_argument("--test_avg_loss", type = int, default = 1, help = "if = 1, calcurate psnr for ave loss")

parser.add_argument("--scene", type = str, default="Planets", help="scene name")
parser.add_argument("--coded_img_path", type=str, default=None, help="path for coded img")
parser.add_argument("--out_depth", type = int ,default=0, help = "if 1, output depth with rendered images")
parser.add_argument("--save_orig_img", type=int, default=0, help="if 1, save 5x5 original image")

args = parser.parse_args()


args.model_num = 1

if args.angRes != 5:
    args.train_aperture = True

args.path_for_train = args.path_for_train + str(args.angRes) + 'x' + str(args.angRes) + '_' +  str(args.patch_size) + '/' + str(args.train_data_var) + '/'