import torch
import torch.backends.cudnn as cudnn
import sys
from util.test_util import NeRFbasedRendering
import torchvision
from pathlib import Path
from PIL import Image

from model import PixelSuffuleReconstruct, VDSR
from model_nerf import create_nerf


def main(args):
    ''' Create Dir for Save '''
    result_dir = Path("Dst")
    result_dir.mkdir(exist_ok=True)

    ''' CPU or Cuda'''
    device = torch.device(args.device)
    if 'cuda' in args.device:
        torch.cuda.set_device(device)


    ''' MODEL LOADING '''
    PSreconstruct= []
    vdsr = []
    for m in range(args.model_num):
            tmp_model = PixelSuffuleReconstruct(args)
            tmp_model.load_state_dict(torch.load("%s/Model/PSreconstruct%01d_Epoch%03d.pth"%(args.model_data,m+1,args.epoch)))
            PSreconstruct.append(tmp_model.to("cuda"))

            tmp_model = VDSR(args, 128)
            tmp_model.load_state_dict(torch.load("%s/Model/vdsr%01d_Epoch%03d.pth"%(args.model_data,m+1,args.epoch)))
            vdsr.append(tmp_model.to("cuda"))

    args.test_mode = 1
    render_kwargs_test = create_nerf(args)
    render_kwargs_test["network_fn"][0].load_state_dict(torch.load("%s/Model/network_fn%01d_Epoch%03d.pth"%(args.model_data, m+1,args.epoch)))


    """Parameters for nerf"""
    near = args.near
    far = args.far
    xy_norm = args.patch_size//2
    bds_dict = {
        'near' : near,
        'far' : far,
        "y_norm" : xy_norm,
        "x_norm" : xy_norm,
        "x_size" : args.patch_size//2,
        "y_size" : args.patch_size//2,
        "z_factor" : args.z_factor
    }
    render_kwargs_test.update(bds_dict)


    cudnn.benchmark = True


    with torch.no_grad():

        save_dir_ = result_dir.joinpath(args.scene)
        save_dir_.mkdir(exist_ok=True)

        nerf_based_rendering = NeRFbasedRendering(args, save_dir_, device)

        if args.coded_img_path is not None:
            """Load coded image and convert to proper format"""
            coded_img = Image.open(args.coded_img_path)
            coded_img = torchvision.transforms.functional.to_tensor(coded_img).to(device)
            coded_img = coded_img * args.shots_num * args.angRes*args.angRes
        
        else:
            print("Please make coded image before running this program.")
            sys.exit(0)


        print("Rendering...")
        """Render 13x13 images from single coded image"""
        nerf_based_rendering.render_imgs(coded_img, PSreconstruct, vdsr, args, **render_kwargs_test)

        print("Scene {} rendered.".format(args.scene))


if __name__ == '__main__':
    from option import args

    main(args)
