import os
import math
import numpy as np
from PIL import Image
import matplotlib.pyplot as plt
import seaborn as sns
import torch

class SaveLog():

        def __init__(self, savedir, savename):
                self.savedir = savedir
                self.savename = savename
                self.savefile = os.path.join(savedir,savename)
                os.makedirs(savedir, exist_ok = True)

        def log(self, savetext):
                with open(self.savefile, mode='a') as f:
                        f.write(savetext)



class SaveParametors():

        def __init__(self, savedir):
                self.savedir = savedir
                self.make_savedirs()

        def make_savedirs(self):
                os.makedirs(os.path.join(self.savedir, "Epoch"), exist_ok=True)
                os.makedirs(os.path.join(self.savedir, "Graph"), exist_ok=True)



        def save_model_param(self, model_list, model_savedir, name, epoch):

                for n in range(len(model_list)):
                        tmp_model = model_list[n].to("cpu")
                        savename = os.path.join(model_savedir,"%s%d_Epoch%03d.pth"%(name, n+1,epoch))
                        # serializers.save_npz(savename, tmp_model)
                        torch.save(tmp_model.state_dict(), savename)
                        model_list[n] = model_list[n].to("cuda")

        def save_optimizer_param(self, optim_list, optim_savedir, name, epoch):

                for n in range(len(optim_list)):
                        tmp_optimzier = optim_list[n]
                        savename = os.path.join(optim_savedir,"%s%d_Epoch%03d.pth"%(name, n+1, epoch))
                        # serializers.save_npz(savename, tmp_optimzier)
                        torch.save(tmp_optimzier.state_dict(), savename)




        def save_models(self, aperture_mask, PSreconstruct, vdsr, forward_pixel_mask, backward_pixel_mask, network_fn, epoch):

                model_savedir = os.path.join(self.savedir, "Epoch/Epoch%03dDone/Model"%epoch)
                os.makedirs(model_savedir, exist_ok=True)
                os.makedirs(os.path.join(model_savedir, "ForwardPixelMask"), exist_ok=True)
                os.makedirs(os.path.join(model_savedir, "BackwardPixelMask"), exist_ok=True)


                self.save_model_param(aperture_mask, model_savedir, "ApertureMask",epoch)
                self.save_model_param(PSreconstruct, model_savedir, "PSreconstruct",epoch)
                self.save_model_param(vdsr, model_savedir, "vdsr",epoch)
                
                self.save_model_param(network_fn, model_savedir, "network_fn", epoch)

                self.save_model_param(forward_pixel_mask, os.path.join(model_savedir, "ForwardPixelMask"), "ForwardPixelMask",epoch)
                self.save_model_param(backward_pixel_mask, os.path.join(model_savedir, "BackwardPixelMask"), "BackwardPixelMask",epoch)



        def save_optimizers(self, aperture_mask_optim, PSreconstruct_optim, vdsr_optim, backward_pixel_mask_optim, network_fn_optim, epoch):

                optimizer_savedir = os.path.join(self.savedir, "Epoch/Epoch%03dDone/Optimizer"%epoch)
                os.makedirs(optimizer_savedir, exist_ok=True)
                # os.makedirs(os.path.join(optimizer_savedir, "ForwardPixelMask"), exist_ok=True)
                os.makedirs(os.path.join(optimizer_savedir, "BackwardPixelMask"), exist_ok=True)

                self.save_optimizer_param(aperture_mask_optim, optimizer_savedir, "ApertureMaskOptimizer",epoch)
                self.save_optimizer_param(PSreconstruct_optim, optimizer_savedir, "PSreconstructOptimizer",epoch)
                self.save_optimizer_param(vdsr_optim, optimizer_savedir, "VdsrOptimizer",epoch)
                self.save_optimizer_param(network_fn_optim, optimizer_savedir, "network_fnOptimizer")

                # self.save_optimizer_param(forward_pixel_mask, os.path.join(optimizer_savedir, "ForwardPixelMask"), "forward_pixel_mask")
                self.save_optimizer_param(backward_pixel_mask_optim, os.path.join(optimizer_savedir, "BackwardPixelMask"), "BackwardPixelMaskOptimizer",epoch)



        def save_mask_image(self, mask_data, savename, image_size=256):
                mask_image = Image.fromarray(np.uint8(mask_data*255.0))
                mask_image = mask_image.resize((image_size, image_size), Image.NEAREST)
                mask_image.save(savename)



        def save_mask_heatmap(self, mask_data, savename):
                plt.figure()
                sns.heatmap(mask_data, vmax = 1.0, vmin = 0.0, annot=True, cmap='gray')
                plt.savefig(savename)
                plt.close(1)



        def save_mask_text(self, mask_data, savename):
                np.savetxt(savename, mask_data, fmt = '%.6e')



        def save_aperture_mask(self, aperture_mask, mask_savedir, mask_savename, image_size=256, Heatmap=True, Text=True):

                image_savedir = os.path.join(mask_savedir, "Mask")
                os.makedirs(image_savedir, exist_ok=True)

                if Heatmap:
                        heatmap_savedir = os.path.join(mask_savedir, "MaskNum")
                        os.makedirs(heatmap_savedir, exist_ok=True)

                if Text:
                        text_savedir = os.path.join(mask_savedir, "Mask/Text")
                        os.makedirs(text_savedir, exist_ok=True)


                for n in range(len(aperture_mask)):
                        # mask_data =  cuda.to_cpu(aperture_mask[n].cn01.W.data[0,:,0,0])
                        mask_data =  aperture_mask[n].cn01.weight.data[0,:,0,0].cpu()
                        mask_size = int(math.sqrt(mask_data.shape[0]))
                        mask_data = mask_data.reshape(mask_size, mask_size)

                        fname = os.path.join(image_savedir, "%s%d.png"%(mask_savename, n+1))
                        self.save_mask_image(mask_data, fname, image_size=image_size)

                        if Heatmap:
                                fname = os.path.join(heatmap_savedir, "%s%d.png"%(mask_savename, n+1))
                                self.save_mask_heatmap(mask_data, fname)

                        if Text:
                                fname = os.path.join(text_savedir, "%sTxt%d.txt"%(mask_savename,n+1))
                                self.save_mask_text(mask_data, fname)



        def save_pixel_mask(self, pixel_mask, shots_num, mask_savedir, mask_savename, image_size=256, Heatmap=True, Text=True):

                image_savedir = os.path.join(mask_savedir, "Mask")
                os.makedirs(image_savedir, exist_ok=True)

                if Heatmap:
                        heatmap_savedir = os.path.join(mask_savedir, "MaskNum")
                        os.makedirs(heatmap_savedir, exist_ok=True)

                if Text:
                        text_savedir = os.path.join(mask_savedir, "Mask/Text")
                        os.makedirs(text_savedir, exist_ok=True)


                for s in range(shots_num):
                        mask_data = np.zeros(len(pixel_mask))
                        for n in range(len(pixel_mask)):
                                # mask_data[n] = cuda.to_cpu(pixel_mask[n].cn01.weight.data[0,s,0,0])
                                mask_data[n] = pixel_mask[n].cn01.weight.data[0,s,0,0].cpu()

                        mask_size = int(math.sqrt(mask_data.shape[0]))
                        mask_data = mask_data.reshape(mask_size, mask_size)

                        fname = os.path.join(image_savedir, "%s%d.png"%(mask_savename, s+1))
                        self.save_mask_image(mask_data, fname, image_size=image_size)

                        if Heatmap:
                                fname = os.path.join(heatmap_savedir, "%s%d.png"%(mask_savename, s+1))
                                self.save_mask_heatmap(mask_data, fname)

                        if Text:
                                fname = os.path.join(text_savedir, "%sTxt%d.txt"%(mask_savename,s+1))
                                self.save_mask_text(mask_data, fname)



        def save_masks(self, aperture_mask, forward_pixel_mask, backward_pixel_mask, epoch):

                mask_savedir = os.path.join(self.savedir, "Epoch/Epoch%03dDone"%epoch)
                os.makedirs(mask_savedir, exist_ok=True)


                self.save_aperture_mask(aperture_mask, mask_savedir, "Epoch%03d_ApertureMask"%(epoch))
                self.save_aperture_mask(aperture_mask, self.savedir, "Epoch%03d_ApertureMask"%(epoch))

                shots_num = len(aperture_mask)
                self.save_pixel_mask(forward_pixel_mask, shots_num, mask_savedir, "Epoch%03d_ForwardPixelMask"%(epoch))
                self.save_pixel_mask(forward_pixel_mask, shots_num, self.savedir, "Epoch%03d_ForwardPixelMask"%(epoch))

                self.save_pixel_mask(backward_pixel_mask, shots_num, mask_savedir, "Epoch%03d_BackwardPixelMask"%(epoch))
                self.save_pixel_mask(backward_pixel_mask, shots_num, self.savedir, "Epoch%03d_BackwardPixelMask"%(epoch))


        def save_graph(self, epoch_index, epoch_psnr, step_index=0, step_psnr=0, epoch=0, frame = 0):
                plt.figure()
                # plt.plot(step_index, step_psnr, label = "Step PSNR")
                plt.plot(epoch_index, epoch_psnr, label = "Epoch PSNR")
                plt.legend()
                plt.grid(True)
                plt.title("PSNR graph")
                plt.xlabel("epoch")
                plt.ylabel("PSNR")

                os.makedirs(os.path.join(self.savedir, "Epoch/Epoch%03dDone/Graph"%epoch), exist_ok = True)
                if frame == 0:
                        fname = os.path.join(self.savedir, "Graph/average_psnr.png")
                        plt.savefig(fname)
                        fname = os.path.join(self.savedir, "Epoch/Epoch%03dDone/Graph/average_psnr.png"%epoch)
                        plt.savefig(fname)
                else:
                        fname = os.path.join(self.savedir, "Graph/Model%02d_psnr.png"%(frame))
                        plt.savefig(fname)
                        fname = os.path.join(self.savedir, "Epoch/Epoch%03dDone/Graph/Model%02d_psnr.png"%(epoch, frame))
                        plt.savefig(fname)

                plt.clf()
                plt.close()
